Package: HDCD
Type: Package
Title: High-Dimensional Changepoint Detection
Version: 1.1
Date: 2024-06-02
Authors@R: 
  c(person(family = "Moen",
           given = "Per August Jarval",
           role = c("cre","aut"),
           email = "pamoen@math.uio.no",
           comment = c(ORCID = "0009-0003-9990-8341")))
Maintainer: Per August Jarval Moen <pamoen@math.uio.no>
Description: Efficient implementations of the following multiple changepoint detection algorithms: Efficient Sparsity Adaptive Change-point estimator by Moen, Glad and Tveten (2023) <doi:10.48550/arXiv.2306.04702> , Informative Sparse Projection for Estimating Changepoints by Wang and Samworth (2017) <doi:10.1111/rssb.12243>, and the method of Pilliat et al (2023) <doi:10.1214/23-EJS2126>.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.2.3
Imports: mclust, Rdpack
RdMacros: Rdpack
NeedsCompilation: yes
Packaged: 2024-06-02 19:34:30 UTC; peraugust
Author: Per August Jarval Moen [cre, aut]
    (<https://orcid.org/0009-0003-9990-8341>)
Repository: CRAN
Date/Publication: 2024-06-02 23:20:26 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 09:57:40 UTC; unix
Archs: HDCD.so.dSYM
