# Generated by roxygen2: do not edit by hand

export(ARI)
export(CUSUM)
export(ESAC)
export(ESAC_calibrate)
export(ESAC_test)
export(ESAC_test_calibrate)
export(Inspect)
export(Inspect_calibrate)
export(Inspect_test)
export(Inspect_test_calibrate)
export(Pilliat)
export(Pilliat_calibrate)
export(Pilliat_test)
export(Pilliat_test_calibrate)
export(hausdorff)
export(rescale_variance)
export(single_CUSUM)
export(single_ESAC)
export(single_Inspect)
export(single_SBS)
export(single_SBS_calibrate)
import(mclust)
import(stats)
importFrom(Rdpack,reprompt)
useDynLib(HDCD,CUSUM_R)
useDynLib(HDCD,cESAC)
useDynLib(HDCD,cESAC_calibrate)
useDynLib(HDCD,cESAC_single)
useDynLib(HDCD,cESAC_test)
useDynLib(HDCD,cESAC_test_calibrate)
useDynLib(HDCD,cInspect)
useDynLib(HDCD,cInspect_calibrate)
useDynLib(HDCD,cInspect_single)
useDynLib(HDCD,cInspect_test_calibrate)
useDynLib(HDCD,cPilliat)
useDynLib(HDCD,cPilliat_calibrate)
useDynLib(HDCD,cPilliat_test)
useDynLib(HDCD,cPilliat_test_calibrate)
useDynLib(HDCD,cSBS_single)
useDynLib(HDCD,cSBS_single_calibrate)
useDynLib(HDCD,partialsum_test)
useDynLib(HDCD,rescale_variance_R)
useDynLib(HDCD,single_CUSUM_R)
useDynLib(HDCD,sort_test)
